@@ -125,12 +125,15 @@ def _checkBuiltin(code, loadValue, argCount, kwArgs, check_arg_count = 1) :
     if loadValue.type == Stack.TYPE_GLOBAL :
         info = python.GLOBAL_FUNC_INFO.get(func_name, None)
         if info is not None :
+            if func_name == 'input' and cfg().usesInput:
+                code.addWarning(msgs.USES_INPUT)
             if kwArgs :
                 code.addWarning(msgs.FUNC_DOESNT_SUPPORT_KW % func_name)
             elif check_arg_count :
                 _checkFunctionArgCount(code, func_name, argCount,
                                        info[1], info[2])
             returnValue = Stack.Item(returnValue.data, info[0])
+            returnValue.setStringType(info[0])
     elif type(func_name) == types.TupleType and len(func_name) <= 2 :
         objType = code.typeMap.get(str(func_name[0]), [])
         if types.ListType in objType :
@@ -151,16 +154,29 @@ def _checkBuiltin(code, loadValue, argCount, kwArgs, check_arg_count = 1) :
                     code.addWarning(msgs.FUNC_DOESNT_SUPPORT_KW % func_name[1])
                 elif methodInfo :
                     returnValue = Stack.Item(func_name[1], methodInfo[0])
+                    returnValue.setStringType(methodInfo[0])
                     if check_arg_count and methodInfo is not None :
                         _checkFunctionArgCount(code, func_name[1], argCount,
                                                methodInfo[1], methodInfo[2])
 
     return returnValue
 
-def _checkModifyDefaultArg(code, objectName) :
+_IMMUTABLE_LIST_METHODS = ('count', 'index',)
+_IMMUTABLE_DICT_METHODS = ('copy', 'get', 'has_key',
+                           'items', 'keys', 'values',
+                           'iteritems', 'iterkeys', 'itervalues')
+
+def _checkModifyDefaultArg(code, objectName, methodName=None) :
     try :
         value = code.func.defaultValue(objectName)
-        if type(value) in python.MUTABLE_TYPES :
+        objectType = type(value)
+        if objectType in python.MUTABLE_TYPES :
+            if objectType == types.DictType and \
+               methodName in _IMMUTABLE_DICT_METHODS :
+                return 
+            if objectType == types.ListType and \
+               methodName in _IMMUTABLE_LIST_METHODS :
+                return
             code.addWarning(msgs.MODIFYING_DEFAULT_ARG % objectName)
     except ValueError :
         pass
@@ -184,6 +200,10 @@ def _checkAbstract(refClass, code, name):
         names = string.join(name_list, ", ")
         code.addWarning(msgs.METHODS_NEED_OVERRIDE % (names, name))
 
+_SEQUENCE_TYPES = (types.TupleType, types.ListType, types.StringType)
+try: _SEQUENCE_TYPES = _SEQUENCE_TYPES + (types.UnicodeType,)
+except AttributeError: pass
+
 def _handleFunctionCall(codeSource, code, argCount, indexOffset = 0,
                         check_arg_count = 1) :
     'Checks for warnings, returns function called (may be None)'
@@ -210,7 +230,7 @@ def _handleFunctionCall(codeSource, code, argCount, indexOffset = 0,
         kwArgs.reverse()
 
     loadValue = code.stack[funcIndex]
-    funcName = loadValue.getName(codeSource.module)
+    funcName = loadValue.getName()
     returnValue = Stack.makeFuncReturnValue(loadValue, argCount)
 
     if loadValue.isMethodCall(codeSource.classObject, cfg().methodArgName) :
@@ -222,7 +242,7 @@ def _handleFunctionCall(codeSource, code, argCount, indexOffset = 0,
         except KeyError :
             sattr = codeSource.classObject.statics.get(methodName)
             if sattr is not None :
-                funcName = sattr.getName(codeSource.classObject)
+                funcName = sattr.getName()
 
             if sattr is None and cfg().callingAttribute :
                 code.addWarning(msgs.INVALID_METHOD % methodName)
@@ -232,11 +252,12 @@ def _handleFunctionCall(codeSource, code, argCount, indexOffset = 0,
         # apply(func, (args)), can't check # of args, so just return func
         if loadValue.data == 'apply' :
             loadValue = code.stack[funcIndex+1]
-            funcName = loadValue.getName(codeSource.module)
+            funcName = loadValue.getName()
         else :
             if cfg().modifyDefaultValue and \
                type(loadValue.data) == types.TupleType :
-                _checkModifyDefaultArg(code, loadValue.data[0])
+                _checkModifyDefaultArg(code, loadValue.data[0],
+                                       loadValue.data[1])
 
             func, refClass, method = _getFunction(codeSource.module, loadValue)
             if func == None and type(loadValue.data) == types.TupleType and \
@@ -257,7 +278,8 @@ def _handleFunctionCall(codeSource, code, argCount, indexOffset = 0,
                 _checkAbstract(refClass, code, funcName)
 
             if func != None :
-                _checkFunctionArgs(code, func, method, argCount, kwArgs, check_arg_count)
+                _checkFunctionArgs(code, func, method, argCount, kwArgs,
+                                   check_arg_count)
                 if refClass :
                     if method :
                         # c'tor, return the class as the type
@@ -302,15 +324,21 @@ def _checkClassAttribute(attr, c, code) :
         code.addWarning(msgs.INVALID_CLASS_ATTR % attr)
 
 def _checkModuleAttribute(attr, module, code, ref) :
-    refModule = module.modules.get(ref)
-    if refModule and refModule.attributes != None :
-        if attr not in refModule.attributes and \
+    try:
+        if attr not in module.modules[ref].attributes and \
            not utils.endswith(ref, '.' + attr) :
             code.addWarning(msgs.INVALID_MODULE_ATTR % attr)
+    except (KeyError, TypeError):
+        # if ref isn't found, or ref isn't even hashable, we don't care
+        # we may not know, or ref could be something funky [e for e].method()
+        pass
 
-    refClass = module.classes.get(ref)
-    if refClass :
-        _checkClassAttribute(attr, refClass, code)
+    try:
+        _checkClassAttribute(attr, module.classes[ref], code)
+    except (KeyError, TypeError):
+        # if ref isn't found, or ref isn't even hashable, we don't care
+        # we may not know, or ref could be something funky [e for e].method()
+        pass
 
 
 def _getGlobalName(name, func) :
@@ -409,6 +437,9 @@ def _handleImport(code, operand, module, main, fromName) :
 
 def _handleImportFrom(code, operand, module, main) :
     fromName = code.stack[-1].data
+    if utils.pythonVersion() < utils.PYTHON_2_0 and \
+       OP.POP_TOP(code.nextOpInfo()[0]):
+        code.popNextOp()
     code.pushStack(Stack.Item(operand, types.ModuleType))
     _handleImport(code, operand, module, main, fromName)
 
@@ -481,6 +512,7 @@ def _getConstant(code, module, data) :
 _UNCHECKABLE_FORMAT_STACK_TYPES = \
       (Stack.TYPE_UNKNOWN, Stack.TYPE_FUNC_RETURN, Stack.TYPE_ATTRIBUTE,
        Stack.TYPE_GLOBAL,)
+_UNCHECKABLE_STACK_TYPES = _UNCHECKABLE_FORMAT_STACK_TYPES + (types.NoneType,)
 
 def _getFormatString(code, codeSource) :
     if len(code.stack) <= 1 :
@@ -559,6 +591,17 @@ def _checkAttributeType(code, stackValue, attr) :
     code.addWarning(msgs.OBJECT_HAS_NO_ATTR % (stackValue.data, attr))
 
 
+def _getTypeStr(t):
+    returnStr = str(t)
+    strs = string.split(returnStr, "'")
+    try:
+        if len(strs) == 3:
+            returnStr = strs[-2]
+    except IndexError:
+        pass
+    return returnStr
+
+
 class Code :
     'Hold all the code state information necessary to find warnings'
 
@@ -571,6 +614,9 @@ class Code :
         self.extended_arg = 0
         self.lastLineNum = 0
         self.maxCode = 0
+        self.has_except = 0
+        self.try_finally_first = 0
+        self.starts_and_ends_with_finally = 0
 
         self.returnValues = []
         self.raiseValues = []
@@ -601,10 +647,16 @@ class Code :
             self.unusedLocals[arg] = 0
             self.typeMap[arg] = [ Stack.TYPE_UNKNOWN ]
 
-    def addWarning(self, err, line = None) :
+    def getWarning(self, err, line = None) :
         if line is None :
             line = self.lastLineNum
-        self.warnings.append(Warning.Warning(self.func_code, line, err))
+        return Warning.Warning(self.func_code, line, err)
+
+    def addWarning(self, err, line = None) :
+        w = err
+        if not isinstance(w, Warning.Warning):
+            w = self.getWarning(err, line)
+        self.warnings.append(w)
 
     def popNextOp(self) :
         self.indexList.append(self.index)
@@ -661,9 +713,7 @@ class Code :
 
     def __getStringStackType(self, data) :
         try :
-            if data.type == types.StringType and not data.const :
-                return Stack.TYPE_UNKNOWN
-            return data.type
+            return data.getType({})
         except AttributeError :
             return Stack.TYPE_UNKNOWN
 
@@ -687,8 +737,27 @@ class Code :
     def setType(self, name) :
         valueList = self.typeMap.get(name, [])
         newType = self.__getStackType()
+        # longs are being merged with ints, assume they are the same
+        # comparisons are really ints anyways
+        if newType in (types.LongType, Stack.TYPE_COMPARISON):
+            newType = types.IntType
         if newType not in valueList :
             valueList.append(newType)
+
+            # need to ignore various types (Unknown, Func return values, etc)
+            # also ignore None, don't care if they use it and a real type
+            if valueList and newType not in _UNCHECKABLE_STACK_TYPES and \
+               cfg().inconsistentTypes:
+                oldTypes = []
+                # only add types to the value list that are "interesting"
+                for typeToAdd in valueList:
+                    if typeToAdd not in _UNCHECKABLE_STACK_TYPES and \
+                       typeToAdd != newType:
+                        oldTypes.append(_getTypeStr(typeToAdd))
+                # do we have any "interesting" old types?  if so, warn
+                if oldTypes:
+                    self.addWarning(msgs.INCONSISTENT_TYPE % \
+                                    (name, oldTypes, _getTypeStr(newType)))
         self.typeMap[name] = valueList
 
     def addReturn(self) :
@@ -702,10 +771,7 @@ class Code :
 
     def addBranch(self, label) :
         if label is not None :
-            try :
-                self.branches[label] = self.branches[label] + 1
-            except KeyError :
-                self.branches[label] = 1
+            self.branches[label] = self.branches.get(label, 0) + 1
 
     def removeBranch(self, label) :
         branch = self.branches.get(label, None)
@@ -798,6 +864,12 @@ def _checkLoadGlobal(codeSource, code, varname) :
                 should_check = 0
 
     if should_check :
+        # if a global var starts w/__ and the global is referenced in a class
+        # we have to strip off the _class-name, to get the original name
+        if codeSource.classObject and \
+           utils.startswith(varname, '_' + codeSource.classObject.name + '__'):
+            varname = varname[len(codeSource.classObject.name)+1:]
+            
         # make sure we remember each global ref to check for unused
         code.globalRefs[_getGlobalName(varname, codeSource.func)] = varname
         if not codeSource.in_class :
@@ -823,7 +895,13 @@ _LOAD_GLOBAL = _LOAD_NAME
 
 def _LOAD_DEREF(oparg, operand, codeSource, code) :
     if type(oparg) == types.IntType :
-        argname = code.func_code.co_varnames[oparg]
+        func_code = code.func_code
+        if codeSource.calling_code :
+            func_code = codeSource.calling_code[-1].function.func_code
+        try:
+            argname = func_code.co_cellvars[oparg]
+        except IndexError:
+            argname = func_code.co_freevars[oparg - len(func_code.co_cellvars)]
         code.pushStack(Stack.Item(argname, types.StringType))
         if code.func_code.co_name != utils.LAMBDA :
             code.unusedLocals[argname] = None
@@ -853,9 +931,12 @@ def _LOAD_CONST(oparg, operand, codeSource, code) :
 
 def _checkLocalShadow(code, module, varname) :
     if module.variables.has_key(varname) :
-        line = module.moduleLineNums.get(varname, (0, 0))[1]
+        line = module.moduleLineNums.get(varname, ('<unknown>', 0))
         if cfg().shadows :
-            code.addWarning(msgs.LOCAL_SHADOWS_GLOBAL % (varname, line))
+            w = code.getWarning(msgs.LOCAL_SHADOWS_GLOBAL % (varname, line[1]))
+            if line[0] != w.file:
+                w.err = '%s in file %s' % (w.err, line[0])
+            code.addWarning(w)
 
 def _checkLoadLocal(code, codeSource, varname, deletedWarn, usedBeforeSetWarn) :
     _checkFutureKeywords(code, varname)
@@ -1053,20 +1134,32 @@ def _pop(oparg, operand, codeSource, code) :
 _POP_TOP = _BINARY_LSHIFT = _BINARY_RSHIFT = \
            _BINARY_AND = _BINARY_XOR = _BINARY_OR = _pop
 
-_NUMERIC_TYPES = (types.IntType, types.FloatType)
+try:
+    ComplexType = types.ComplexType
+except NameError:
+    ComplexType = types.FloatType    # need some numeric type here
+
+_NUMERIC_TYPES = (types.IntType, types.FloatType, ComplexType)
 
 # FIXME: This is pathetically weak, need to handle more types
 def _coerce_type(code) :
     if OP.POP_TOP(code.nextOpInfo()[0]) :
         code.addWarning(msgs.POSSIBLE_STMT_WITH_NO_EFFECT)
 
+    newItem = Stack.Item('<stack>', Stack.TYPE_UNKNOWN)
     if len(code.stack) >= 2 :
         s1, s2 = code.stack[-2:]
         s1type = s1.getType(code.typeMap)
         s2type = s2.getType(code.typeMap)
         if s1type != s2type :
             if s1type in _NUMERIC_TYPES and s2type in _NUMERIC_TYPES :
-                code.setType(code.stack[-2].data)
+                newType = types.FloatType
+                if s1type == ComplexType or s2type == ComplexType:
+                    newType = ComplexType
+                newItem.type = newType
+
+    code.popStackItems(2)
+    code.pushStack(newItem)
 
 def _BINARY_ADD(oparg, operand, codeSource, code) :
     stack = code.stack
@@ -1077,14 +1170,9 @@ def _BINARY_ADD(oparg, operand, codeSource, code) :
         code.pushStack(Stack.Item(value, type(value), 1))
     else :
         _coerce_type(code)
-        code.popStack()
-        if stack :
-            # we know that the stack can't be const
-            stack[-1].const = 0
 
 def _BINARY_SUBTRACT(oparg, operand, codeSource, code) :
     _coerce_type(code)
-    code.popStack()
 _BINARY_POWER = _BINARY_SUBTRACT
 
 def _BINARY_SUBSCR(oparg, operand, codeSource, code) :
@@ -1121,10 +1209,11 @@ def _BINARY_MULTIPLY(oparg, operand, codeSource, code) :
         format = _getFormatString(code, codeSource)
         if format and type(code.stack[-1].data) == types.IntType :
             code.stack[-2].data = format * code.stack[-1].data
-
-        _coerce_type(code)
-
-    code.popStack()
+            code.popStack()
+        else:
+            _coerce_type(code)
+    else:
+        code.popStack()
 
 def _BINARY_MODULO(oparg, operand, codeSource, code) :
     if OP.POP_TOP(code.nextOpInfo()[0]) :
@@ -1132,20 +1221,50 @@ def _BINARY_MODULO(oparg, operand, codeSource, code) :
 
     _getFormatWarnings(code, codeSource)
     code.popStack()
+    if code.stack:
+        code.stack[-1].const = 0
 
 def _ROT_TWO(oparg, operand, codeSource, code) :
     if len(code.stack) >= 2 :
         del code.stack[-2]
 
 def _SETUP_EXCEPT(oparg, operand, codeSource, code) :
+    code.has_except = 1
     code.pushStack(Stack.Item(None, Stack.TYPE_EXCEPT))
     code.pushStack(Stack.Item(None, Stack.TYPE_EXCEPT))
 
+def _SETUP_FINALLY(oparg, operand, codeSource, code) :
+    if not code.has_except :
+        code.try_finally_first = 1
+
+def _END_FINALLY(oparg, operand, codeSource, code) :
+    if code.try_finally_first and code.index == (len(code.bytes) - 4) :
+        code.starts_and_ends_with_finally = 1
+
 def _LINE_NUM(oparg, operand, codeSource, code) :
     code.lastLineNum = oparg
+
 def _UNPACK_SEQUENCE(oparg, operand, codeSource, code) :
     code.unpackCount = oparg
-
+    if code.stack:
+        top = code.stack[-1]
+        # if we know we have a tuple, make sure we unpack it into the
+        # right # of variables
+        topType = top.getType(code.typeMap)
+        if topType in _SEQUENCE_TYPES:
+            length = top.length
+            # we don't know the length, maybe it's constant and we can find out
+            if length == 0:
+                value = code.constants.get(str(top.data))
+                if type(value) in _SEQUENCE_TYPES:
+                    length = len(value)
+            if length > 0 and length != oparg:
+                if cfg().unpackLength:
+                    code.addWarning(msgs.WRONG_UNPACK_SIZE % (length, oparg))
+        elif topType not in _UNCHECKABLE_STACK_TYPES:
+            if cfg().unpackNonSequence:
+                code.addWarning(msgs.UNPACK_NON_SEQUENCE %
+                                (top.data, _getTypeStr(topType)))
 
 def _SLICE_1_ARG(oparg, operand, codeSource, code) :
     _popStackRef(code, operand)
@@ -1155,10 +1274,26 @@ _SLICE1 = _SLICE2 = _SLICE_1_ARG
 def _SLICE3(oparg, operand, codeSource, code) :
     _popStackRef(code, operand, 3)
 
+def _check_string_iteration(code, index):
+    try:
+        item = code.stack[index]
+    except IndexError:
+        return
+    if item.getType(code.typeMap) == types.StringType and \
+       cfg().stringIteration:
+        code.addWarning(msgs.STRING_ITERATION % item.data)
+
 def _FOR_LOOP(oparg, operand, codeSource, code) :
     code.loops = code.loops + 1
+    _check_string_iteration(code, -2)
     _popStackRef(code, '<for_loop>', 2)
-_FOR_ITER = _FOR_LOOP
+
+def _GET_ITER(oparg, operand, codeSource, code) :
+    _check_string_iteration(code, -1)
+
+def _FOR_ITER(oparg, operand, codeSource, code) :
+    code.loops = code.loops + 1
+    _popStackRef(code, '<for_iter>', 1)
 
 def _jump(oparg, operand, codeSource, code) :
     if len(code.stack) > 0 :
@@ -1169,6 +1304,22 @@ def _jump(oparg, operand, codeSource, code) :
                 code.addWarning(msgs.USING_METHOD_AS_ATTR % name)
 _JUMP_ABSOLUTE = _jump
 
+def _skip_loops(bytes, i, lastLineNum, max) :
+    extended_arg = 0
+    blockCount = 1
+    while i < max :
+        op, oparg, i, extended_arg = OP.getInfo(bytes, i, extended_arg)
+        if OP.LINE_NUM(op) :
+            lastLineNum = oparg
+        elif OP.FOR_LOOP(op) or OP.FOR_ITER(op) or OP.SETUP_LOOP(op) :
+            blockCount = blockCount + 1
+        elif OP.POP_BLOCK(op) :
+            blockCount = blockCount - 1
+            if blockCount <= 0 :
+                break
+
+    return lastLineNum, i
+
 def _is_unreachable(code, topOfStack, branch, if_false) :
     # Are we are checking exceptions, but we not catching all exceptions?
     if (topOfStack.type == Stack.TYPE_COMPARISON and 
@@ -1188,10 +1339,22 @@ def _is_unreachable(code, topOfStack, branch, if_false) :
 
     # check if we break out of the loop
     i = code.index
+    lastLineNum = code.lastLineNum
     while i < branch :
         op, oparg, i, extended_arg = OP.getInfo(code.bytes, i, extended_arg)
-        if OP.BREAK_LOOP(op) :
+        if OP.LINE_NUM(op) :
+            lastLineNum = oparg
+        elif OP.BREAK_LOOP(op) :
             return 0
+        elif OP.FOR_LOOP(op) or OP.FOR_ITER(op) or OP.SETUP_LOOP(op) :
+            lastLineNum, i = _skip_loops(code.bytes, i, lastLineNum, branch)
+
+    i = code.index - 3*4
+    op, oparg, i, extended_arg = OP.getInfo(code.bytes, i, 0)
+    if OP.SETUP_LOOP(op) :
+        # a little lie to pretend we have a raise after a while 1:
+        code.removeBranch(i + oparg)
+        code.raiseValues.append((lastLineNum, None, i + oparg))
     return 1
 
 def _jump_conditional(oparg, operand, codeSource, code, if_false) :
@@ -1201,9 +1364,9 @@ def _jump_conditional(oparg, operand, codeSource, code, if_false) :
            (topOfStack.data != 1 or cfg().constant1) :
             code.addWarning(msgs.CONSTANT_CONDITION % str(topOfStack))
 
-        if _is_unreachable(code, topOfStack, code.index + oparg, if_false) :
-            code.removeBranch(code.index + oparg)
-                
+        if _is_unreachable(code, topOfStack, code.label, if_false) :
+            code.removeBranch(code.label)
+
     _jump(oparg, operand, codeSource, code)
 
 def _JUMP_IF_FALSE(oparg, operand, codeSource, code) :
@@ -1256,9 +1419,11 @@ DISPATCH[ 63] = _BINARY_RSHIFT
 DISPATCH[ 64] = _BINARY_AND
 DISPATCH[ 65] = _BINARY_XOR
 DISPATCH[ 66] = _BINARY_OR
+DISPATCH[ 68] = _GET_ITER
 DISPATCH[ 83] = _RETURN_VALUE
 DISPATCH[ 84] = _IMPORT_STAR
 DISPATCH[ 85] = _EXEC_STMT
+DISPATCH[ 88] = _END_FINALLY
 DISPATCH[ 90] = _STORE_NAME
 DISPATCH[ 91] = _DELETE_NAME
 DISPATCH[ 92] = _UNPACK_SEQUENCE
@@ -1283,6 +1448,7 @@ DISPATCH[113] = _JUMP_ABSOLUTE
 DISPATCH[114] = _FOR_LOOP
 DISPATCH[116] = _LOAD_GLOBAL
 DISPATCH[121] = _SETUP_EXCEPT
+DISPATCH[122] = _SETUP_FINALLY
 DISPATCH[124] = _LOAD_FAST
 DISPATCH[125] = _STORE_FAST
 DISPATCH[126] = _DELETE_FAST
