@@ -28,6 +28,7 @@ class Item :
         self.type = dataType
         self.const = const
         self.length = length
+        self.is_really_string = 0
 
     def __str__(self) :
         if type(self.data) == types.TupleType :
@@ -53,8 +54,11 @@ class Item :
     def isLocals(self) :
         return self.type == types.DictType and self.data == LOCALS
 
+    def setStringType(self, value = types.StringType):
+        self.is_really_string = value == types.StringType
+
     def getType(self, typeMap) :
-        if self.type != types.StringType :
+        if self.type != types.StringType or self.is_really_string:
             return self.type
         if self.const :
             return type(self.data)
@@ -64,24 +68,12 @@ class Item :
                 return localTypes[0]
         return TYPE_UNKNOWN
 
-    def getName(self, module) :
-        if self.type == TYPE_ATTRIBUTE and \
-           type(self.data) != types.StringType :
-            strValue, data = "", self.data
-            # handle:  from XXX import YYY to munge the name 
-            # into looking like: XXX.YYY
-            if type(self.data[0]) == types.StringType and \
-               hasattr(module.module, self.data[0]) :
-                globalObject = getattr(module.module, self.data[0])
-                data = self.data[1:]
-                if type(globalObject) != types.ModuleType :
-                    strValue = "." + str(globalObject)
-                else :
-                    strValue = "." + globalObject.__name__
-
+    def getName(self) :
+        if self.type == TYPE_ATTRIBUTE and type(self.data) != types.StringType:
+            strValue = ""
             # convert the tuple into a string ('self', 'data') -> self.data
-            for item in data :
-                strValue = strValue + "." + str(item)
+            for item in self.data :
+                strValue = '%s.%s' % (strValue, str(item))
             return strValue[1:]
         return str(self.data)
 
