@@ -7,7 +7,7 @@ Check python source code files for possible errors and print warnings
 
 Contact Info:
   http://pychecker.sourceforge.net/
-  pychecker@metaslash.com
+  pychecker-list@lists.sourceforge.net
 """
 
 import string
@@ -16,6 +16,7 @@ import sys
 import imp
 import os
 import glob
+import traceback
 
 # see __init__.py for meaning, this must match the version there
 LOCAL_MAIN_VERSION = 1
@@ -98,17 +99,18 @@ def getModules(arg_list) :
 
             module_name = os.path.basename(arg)[:-PY_SUFFIX_LEN]
             if arg_dir not in sys.path :
-                sys.path.insert(1, arg_dir)
+                sys.path.insert(0, arg_dir)
             arg = module_name
         modules.append(arg)
 
     return modules
 
 
-def _findModule(name, path = sys.path) :
+def _findModule(name) :
     """Returns the result of an imp.find_module(), ie, (file, filename, smt)
        name can be a module or a package name.  It is *not* a filename."""
 
+    path = sys.path[:]
     packages = string.split(name, '.')
     for p in packages :
         # smt = (suffix, mode, type)
@@ -137,7 +139,7 @@ def _findModule(name, path = sys.path) :
                 new_path = filename
             if new_path not in path :
                 path.insert(1, new_path)
-        else:
+        elif smt[-1] != imp.PY_COMPILED:
             if p is not packages[-1] :
                 if file is not None :
                     file.close()
@@ -156,6 +158,11 @@ class Variable :
         self.type = type
         self.value = None
 
+    def __str__(self) :
+        return self.name
+
+    __repr__ = utils.std_repr
+
 
 def _filterDir(object, ignoreList) :
     "Return a list of tokens (attributes) in a class, except for ignoreList"
@@ -175,8 +182,16 @@ class Class :
 
     def __init__(self, name, module) :
         self.name = name
-        self.module = module
         self.classObject = getattr(module, name)
+
+        # Argh -- some ExtensionClasses don't have a __module__ attribute.
+        modname = getattr(self.classObject, '__module__', None)
+        self.module = sys.modules.get(modname)
+        if not self.module:
+            self.module = module
+            sys.stderr.write("warning: couldn't find real module for class %r "
+                             "(module name: %r)\n"
+                             % (self.classObject, modname))
         self.ignoreAttrs = 0
         self.methods = {}
         self.members = { '__class__': types.ClassType,
@@ -186,6 +201,11 @@ class Class :
         self.statics = {}
         self.lineNums = {}
 
+    def __str__(self) :
+        return self.name
+
+    __repr__ = utils.std_repr
+
     def getFirstLine(self) :
         "Return first line we can find in THIS class, not any base classes"
 
@@ -282,7 +302,7 @@ class Class :
 
     def cleanupMemberRefs(self) :
         try :
-            del self.memberRefs['__pychecker__']
+            del self.memberRefs[Config.CHECKER_VAR]
         except KeyError :
             pass
 
@@ -312,11 +332,41 @@ class Class :
                 result.append(m)
         return result
 
-def importError(moduleName, info):
-    # detail may contain a newline replace with - 
-    # use str to avoid undestanding the tuple structure in the exception
-    info = string.join(string.split(str(info), '\n' ), ' - ')
-    sys.stderr.write("  Problem importing module %s - %s\n" % (moduleName, info))
+def importError(moduleName):
+    exc_type, exc_value, tb = sys.exc_info()
+
+    # First, try to get a nice-looking name for this exception type.
+    exc_name = getattr(exc_type, '__name__', None)
+    if not exc_name:
+        # either it's a string exception or a user-defined exception class
+        # show string or fully-qualified class name
+        exc_name = str(exc_type)
+        
+    # Print a traceback, unless this is an ImportError.  ImportError is
+    # presumably the most common import-time exception, so this saves
+    # the clutter of a traceback most of the time.  Also, the locus of
+    # the error is usually irrelevant for ImportError, so the lack of
+    # traceback shouldn't be a problem.
+    if exc_type is not ImportError:
+        sys.stderr.write("  Caught exception importing module %s:\n" %
+                         moduleName)
+
+        tbinfo = traceback.extract_tb(tb)
+        for filename, line, func, text in tbinfo[1:]:
+            sys.stderr.write("    File \"%s\", line %d" % (filename, line))
+            if func != "?":
+                sys.stderr.write(", in %s()" % func)
+            sys.stderr.write("\n")
+            if text:
+                sys.stderr.write("      %s\n" % text)
+
+    # And finally print the exception type and value.
+    # Careful formatting exc_value -- can fail for some user exceptions
+    sys.stderr.write("  %s: " % exc_name)
+    try:
+        sys.stderr.write(str(exc_value) + '\n')
+    except:
+        sys.stderr.write('**error formatting exception value**\n')
 
 
 class Module :
@@ -335,6 +385,11 @@ class Module :
         self.check = check
         _allModules[moduleName] = self
 
+    def __str__(self) :
+        return self.moduleName
+
+    __repr__ = utils.std_repr
+
     def addVariable(self, var, varType) :
         self.variables[var] = Variable(var, varType)
 
@@ -392,7 +447,8 @@ class Module :
             exc_type, exc_value, exc_tb = sys.exc_info()
             raise exc_type, exc_value
         except :
-            return importError(self.moduleName, sys.exc_info()[1])
+            importError(self.moduleName)
+            return 0
 
     def initModule(self, module) :
         self.module = module
@@ -565,9 +621,9 @@ else :
                     _printWarnings(warn.find([module], _cfg, _suppressions))
                 else :
                     print 'Unable to load module', pymodule.__name__
-            except Exception, detail:
+            except Exception:
                 name = getattr(pymodule, '__name__', str(pymodule))
-                print 'PyChecker error processing %s: %s' % (name, detail)
+                importError(name)
 
         return pymodule
 
