@@ -66,8 +66,10 @@ TOO_MANY_REFERENCES = 'Law of Demeter violated, more than %d references for (%s)
 
 IMPLICIT_AND_EXPLICIT_RETURNS = "Function returns a value and also implicitly returns None"
 INCONSISTENT_RETURN_TYPE = "Function return types are inconsistent"
+INCONSISTENT_TYPE = "Variable (%s) already has types %s and set to %s"
 CODE_UNREACHABLE = "Code appears to be unreachable"
 CONSTANT_CONDITION = "Using a conditional statement with a constant value (%s)"
+STRING_ITERATION = "Iterating over a string (%s)"
 DONT_RETURN_NONE = "%s should not return None, raise an exception if not found"
 
 INVALID_FORMAT = "Invalid format string, problem starts near: '%s'"
@@ -93,6 +95,8 @@ USING_KEYWORD = "Using identifier (%s) which will become a keyword in version %s
 MODIFYING_DEFAULT_ARG = "Modifying parameter (%s) with a default value may have unexpected consequences"
 USING_SELF_IN_REPR = "Using `self` in __repr__ method"
 USING_NONE_RETURN_VALUE = "Using the return value from (%s) which is always None"
+WRONG_UNPACK_SIZE = "Unpacking %d values into %d variables"
+UNPACK_NON_SEQUENCE = "Unpacking a non-sequence (%s) of type %s"
 
 USING_PROPERTIES_IN_CLASSIC_CLASS = "Using property (%s) in classic class %s may not work"
 USING_SLOTS_IN_CLASSIC_CLASS = "Using __slots__ in classic class %s is not used by Python"
@@ -100,3 +104,4 @@ EMPTY_SLOTS = "__slots__ are empty in %s"
 
 USES_EXEC = "Using the exec statement"
 USES_GLOBAL_EXEC = "Using the exec statement in global namespace"
+USES_INPUT = "Using input() is a security problem, consider using raw_input()"
