@@ -22,7 +22,7 @@ GLOBAL_FUNC_INFO = { '__import__': (types.ModuleType, 1, 4),
                      'buffer': (types.BufferType, 1, 3),
                      'callable': (BOOL, 1, 1),
                      'chr': (types.StringType, 1, 1),
-                     'cmp': (BOOL, 2, 2),
+                     'cmp': (types.IntType, 2, 2),
                      'coerce': ([ types.NoneType, types.TupleType ], 2, 2),
                      'compile': (types.CodeType, 3, 3),
                      'complex': (types.ComplexType, 1, 2),
