@@ -6,10 +6,12 @@
 Print out warnings from Python source files.
 """
 
+import os.path
 import sys
 import string
 import types
 import traceback
+import imp
 
 from pychecker import OP
 from pychecker import Stack
@@ -73,15 +75,17 @@ def _checkReturnWarnings(code) :
 
     # if the last return is implicit, check if there are non None returns
     lastReturn = code.returnValues[-1]
-    if cfg().checkImplicitReturns and lastReturn[1].isImplicitNone() :
+    if not code.starts_and_ends_with_finally and \
+       cfg().checkImplicitReturns and lastReturn[1].isImplicitNone() :
         for line, retval, dummy in code.returnValues[:-1] :
             if not retval.isNone() :
                 code.addWarning(msgs.IMPLICIT_AND_EXPLICIT_RETURNS,
                                 lastReturn[0]+1)
                 break
 
-    # getattr can return different types, so don't warn about inconsistency
-    if is_getattr :
+    # __get*__ funcs can return different types, don't warn about inconsistency
+    if utils.startswith(code.func_code.co_name, '__get') and \
+       utils.endswith(code.func_code.co_name, '__') :
         return
 
     returnType, returnData = None, None
@@ -246,6 +250,7 @@ def _checkFunction(module, func, c = None, main = 0, in_class = 0) :
 
     if not (main or in_class) :
         utils.popConfig()
+    func.returnValues = code.returnValues
     return (code.warnings, code.globalRefs, code.functionsCalled,
             code.codeObjects.values(), code.returnValues)
 
@@ -257,15 +262,22 @@ def _getUnused(module, globalRefs, dict, msg, filterPrefix = None) :
     for ref in dict.keys() :
         check = not filterPrefix or utils.startswith(ref, filterPrefix)
         if check and globalRefs.get(ref) == None :
-            lineInfo = module.moduleLineNums.get(ref, (module.filename(), 1))
-            warnings.append(Warning(lineInfo[0], lineInfo[1], msg % ref))
+            lineInfo = module.moduleLineNums.get(ref)
+            if lineInfo:
+                warnings.append(Warning(lineInfo[0], lineInfo[1], msg % ref))
     return warnings
 
 
 def _get_func_info(method) :
-    fc = getattr(method.im_func, 'func_code', None)
-    if fc is not None :
-        return fc.co_filename, fc.co_firstlineno
+    try:
+        fc = getattr(method.im_func, 'func_code', None)
+        if fc is not None :
+            return fc.co_filename, fc.co_firstlineno
+    except AttributeError:
+        # if the object derives from any object in 2.2,
+        # the builtin methods are wrapper_descriptors and
+        # have no im_func attr
+        pass
     return None, None
 
 _DOT_INIT = '.' + utils.INIT
@@ -345,7 +357,6 @@ def getBlackList(moduleList) :
         if badBoy[-3:] == ".py":
             badBoy = badBoy[0:-3]
         try :
-            import imp
             file, path, flags = imp.find_module(badBoy)
             if file :
                 file.close()
@@ -357,7 +368,6 @@ def getBlackList(moduleList) :
 def getStandardLibrary() :
     if cfg().ignoreStandardLibrary :
         try :
-            import os.path
             from distutils import sysconfig
 
             std_lib = sysconfig.get_python_lib()
@@ -510,10 +520,9 @@ def _findClassWarnings(module, c, class_code,
             err = msgs.EMPTY_SLOTS % c.name
             warnings.append(Warning(filename, lineNum, err))
 
-    if not newStyleClass and property is not None and \
-       cfg().classicProperties:
+    if not newStyleClass and property is not None and cfg().classicProperties:
         for static in c.statics.keys():
-            if type(getattr(c.classObject, static)) == property:
+            if type(getattr(c.classObject, static, None)) == property:
                 err = msgs.USING_PROPERTIES_IN_CLASSIC_CLASS % (static, c.name)
                 warnings.append(Warning(filename, c.lineNums[static], err))
 
