@@ -1,7 +1,7 @@
 "doesn't work in 1.5.2"
 
 def ttt(c):
-  pass
+  return None
 
 def x():
   "should not complain about either, we can't check # args"
